/**
 * @file        ADSO-50212
 * @description Batch edit forecasts
 * @author      Clarence (ccn7@3ds.com)
 * @copyright   Dassault Systèmes
 */
import { AppMP, Demo, Scenario } from '../../libmp/appmp';
import { DataMetalBaseProductName } from '../../libmp/data/data.product';
import { DataMetalBaseSalesSegmentName } from '../../libmp/data/data.salessegment';
import { DataMetalStockingPointName } from '../../libmp/data/data.stockingpoint';
import { ListForecastsColumn, ListForecastsContextMenuItem } from '../../libmp/forms/form.forecasts';
import { DialogForecast, DialogForecastFields } from '../../libmp/dialogs/dialog.forecast';
import { DialogBatchEditActionLink, StepDialog } from '../../libappsop/dialogsop';
import { StepList } from '../../libappsop/listsop';
import { StepNavigationPanel } from '../../libmp/forms/navigationpanel/form.navigationpanel';
import { startOfPlanningYear } from '../../libmp/data/data.period';

describe('ADSO-50212 - Batch edit forecasts', () => {
  const appMP = AppMP.getInstance();
  const listForecasts = appMP.viewForecasts.formForecasts.listForecasts;
  const janSOP = `1-Jan-${startOfPlanningYear}`;
  const febSOP = `1-Feb-${startOfPlanningYear}`;
  const aprSOP = `1-Apr-${startOfPlanningYear}`;
  let dlgForecast: DialogForecast;
  const editRows: ListForecastsColumn[] = [{'Sales segment': DataMetalBaseSalesSegmentName.AllOrganics, Product: DataMetalBaseProductName.BCCBNarrow, 'Stocking point': DataMetalStockingPointName.NorthAmerica, Start: janSOP},
                                           {'Sales segment': DataMetalBaseSalesSegmentName.AllOrganics, Product: DataMetalBaseProductName.BCCBNarrow, 'Stocking point': DataMetalStockingPointName.NorthAmerica, Start: febSOP}];
  const updateDlgBatchValues: DialogForecastFields = {SalesSegment: DataMetalBaseSalesSegmentName.AudiA8, Product: DataMetalBaseProductName.BCCanBody, End: aprSOP, OriginalQuantity: '9999'};
  const verifyRow1: ListForecastsColumn = {'Sales segment': DataMetalBaseSalesSegmentName.AudiA8, Product: DataMetalBaseProductName.BCCanBody, 'Stocking point': DataMetalStockingPointName.NorthAmerica, Start: janSOP, End: aprSOP, 'Original Qty': '9,999'};
  const verifyRow2: ListForecastsColumn = {'Sales segment': DataMetalBaseSalesSegmentName.AudiA8, Product: DataMetalBaseProductName.BCCanBody, 'Stocking point': DataMetalStockingPointName.NorthAmerica, Start: febSOP, End: aprSOP, 'Original Qty': '9,999'};

  beforeAll(async () => {
    await appMP.login();
  });

  afterAll(async () => {
    await appMP.resetActiveView(appMP.viewForecasts);
    await appMP.cleanupAndLogout();
  });

  afterEach(async () => {
    await appMP.checkToastMessage();
  });

  it(`Step 1 - ${AppMP.getDemoDataPath(Demo.Metals, Scenario.Base)}.`, async () => {
    await appMP.createDemoDataset(Demo.Metals, Scenario.Base, false);
  });

  it(`Step 2 - Open view ${appMP.viewForecasts.viewPath}.`, async () => {
    await appMP.viewForecasts.switchTo();
  });

  it(`Step 3 - ${StepList.selectRowsAndClickMenu(listForecasts.title, editRows, ListForecastsContextMenuItem.Edit.Label)}`, async () => {
    const rows = await listForecasts.selectListRows(editRows);

    [dlgForecast] = await listForecasts.selectContextMenu(ListForecastsContextMenuItem.Edit, rows);
  });

  it(`Step 4 - ${StepDialog.verifyBatchEditEnabled(DialogForecast.title, {SalesSegment: 'false', Product: 'false', StockingPoint: 'false', Start: 'false', End: 'false', Price: 'false', OriginalQuantity: 'false'})}`, async () => {
    await dlgForecast.verifyBatchEditEnabled({SalesSegment: 'false', Product: 'false', StockingPoint: 'false', Start: 'false', End: 'false', Price: 'false', OriginalQuantity: 'false'});
  });

  it(`Step 5 - ${StepDialog.updateDialogValuesBatchEdit(DialogForecast.title, DialogBatchEditActionLink.EnableBatchEdit, updateDlgBatchValues)}`, async () => {
    await dlgForecast.updateDialogValuesBatchEdit(DialogBatchEditActionLink.EnableBatchEdit, updateDlgBatchValues);
  });

  it(`Step 6 - ${StepDialog.clickOK()}`, async () => {
    await dlgForecast.clickOK();
  });

  it(`Step 7 - ${StepNavigationPanel.showProductsList()} ${StepNavigationPanel.checkProducts([DataMetalBaseProductName.BCCanBody])}`, async () => {
    await appMP.formNavigation.toggleProductList(true);

    const row = await appMP.formNavigation.listProduct.getRow({Name: DataMetalBaseProductName.BCCanBody}, [{Name: DataMetalBaseProductName.AllProducts}, {Name: DataMetalBaseProductName.Can}, {Name: DataMetalBaseProductName.BeverageCan}]);
    await appMP.formNavigation.listProduct.toggleRowCheckbox(row, true);
  });

  it(`Step 8 - ${StepNavigationPanel.showSalesSegmentsList()} ${StepNavigationPanel.checkSalesSegments([DataMetalBaseSalesSegmentName.Automotive])}`, async () => {
    await appMP.formNavigation.toggleSalesSegmentList(true);

    const row = await appMP.formNavigation.listSalesSegment.getRow({Name: DataMetalBaseSalesSegmentName.Automotive}, [{Name: DataMetalBaseSalesSegmentName.AllSalesSegments}]);
    await appMP.formNavigation.listSalesSegment.toggleRowCheckbox(row, true);
  });

  it(`Step 9 - ${StepList.verifyRowExists(listForecasts.title, verifyRow1)} ${StepList.verifyRowExists(listForecasts.title, verifyRow2)}`, async () => {
    await listForecasts.verifyRowExists(verifyRow1);
    await listForecasts.verifyRowExists(verifyRow2);
  });
});
